@@ -248,6 +248,134 @@ static void output_distances(struct lstopo_output *loutput)
   free(dist);
 }
 
+static void output_memattr_initiator(struct lstopo_output *loutput,
+                                     struct hwloc_location *initiator)
+{
+  hwloc_topology_t topology = loutput->topology;
+  enum lstopo_index_type_e index_type = loutput->index_type;
+
+  if (initiator->type == HWLOC_LOCATION_TYPE_CPUSET) {
+    hwloc_obj_t obj;
+    char *c;
+
+    assert(initiator->location.cpuset);
+    hwloc_bitmap_asprintf(&c, initiator->location.cpuset);
+    printf(" from cpuset %s", c);
+    free(c);
+
+    obj = hwloc_get_obj_covering_cpuset(topology, initiator->location.cpuset);
+    if (obj && !hwloc_bitmap_isequal(obj->cpuset, initiator->location.cpuset))
+      obj = NULL;
+    if (obj) {
+      char objtype[16];
+      while (obj->parent && hwloc_bitmap_isequal(obj->cpuset, obj->parent->cpuset))
+        obj = obj->parent;
+      hwloc_obj_type_snprintf(objtype, sizeof(objtype), obj, 0);
+      printf(" (%s %c#%u)", objtype,
+             index_type == LSTOPO_INDEX_TYPE_PHYSICAL ? 'P' : 'L',
+             index_type == LSTOPO_INDEX_TYPE_PHYSICAL ? obj->os_index : obj->logical_index);
+    }
+
+  } else if (initiator->type == HWLOC_LOCATION_TYPE_OBJECT) {
+    hwloc_obj_t obj = initiator->location.object;
+    char objtype[16];
+    assert(obj);
+    hwloc_obj_type_snprintf(objtype, sizeof(objtype), obj, 0);
+    printf(" (%s %c#%u)", objtype,
+           index_type == LSTOPO_INDEX_TYPE_PHYSICAL ? 'P' : 'L',
+           index_type == LSTOPO_INDEX_TYPE_PHYSICAL ? obj->os_index : obj->logical_index);
+
+  } else {
+    printf(" from initiator with unexpected type %d\n",
+	   (int) initiator->type);
+  }
+
+}
+
+static void output_memattrs(struct lstopo_output *loutput)
+{
+  hwloc_topology_t topology = loutput->topology;
+  enum lstopo_index_type_e index_type = loutput->index_type;
+  int verbose_mode = loutput->verbose_mode;
+  int show_all = (loutput->show_memattrs_only || (verbose_mode >= 3));
+  unsigned id;
+
+  for(id=0; ; id++) {
+    const char *name;
+    unsigned long flags;
+    unsigned nr_targets;
+    hwloc_obj_t *targets;
+    unsigned i;
+    int err;
+
+    if (!show_all
+        && (id == HWLOC_MEMATTR_ID_CAPACITY || id == HWLOC_MEMATTR_ID_LOCALITY))
+      continue;
+
+    err = hwloc_memattr_get_name(topology, id, &name);
+    if (err < 0)
+      break;
+    err = hwloc_memattr_get_flags(topology, id, &flags);
+    assert(!err);
+
+    nr_targets = 0;
+    err = hwloc_memattr_get_targets(topology, id, NULL, 0, &nr_targets, NULL, NULL);
+    assert(!err);
+
+    if (!show_all && !nr_targets)
+      continue;
+
+    printf("Memory attribute #%u name `%s' flags %lu\n", id, name, flags);
+
+    targets = malloc(nr_targets * sizeof(*targets));
+    if (!targets)
+      continue;
+
+    err = hwloc_memattr_get_targets(topology, id, NULL, 0, &nr_targets, targets, NULL);
+    assert(!err);
+
+    for(i=0; i<nr_targets; i++) {
+
+      if (!(flags & HWLOC_MEMATTR_FLAG_NEED_INITIATOR)) {
+        hwloc_uint64_t value;
+        err = hwloc_memattr_get_value(topology, id, targets[i], NULL, 0, &value);
+        if (!err)
+          printf("  %s %c#%u = %llu\n",
+                 hwloc_obj_type_string(targets[i]->type),
+                 index_type == LSTOPO_INDEX_TYPE_PHYSICAL ? 'P' : 'L',
+                 index_type == LSTOPO_INDEX_TYPE_PHYSICAL ? targets[i]->os_index : targets[i]->logical_index,
+                 (unsigned long long) value);
+
+      } else {
+        unsigned nr_initiators = 0;
+        err = hwloc_memattr_get_initiators(topology, id, targets[i], 0, &nr_initiators, NULL, NULL);
+        if (!err) {
+          struct hwloc_location *initiators = malloc(nr_initiators * sizeof(*initiators));
+          hwloc_uint64_t *values = malloc(nr_initiators * sizeof(*values));
+          if (initiators && values) {
+            err = hwloc_memattr_get_initiators(topology, id, targets[i], 0, &nr_initiators, initiators, values);
+            if (!err) {
+              unsigned j;
+              for(j=0; j<nr_initiators; j++) {
+                printf("  %s %c#%u = %llu",
+                       hwloc_obj_type_string(targets[i]->type),
+                       index_type == LSTOPO_INDEX_TYPE_PHYSICAL ? 'P' : 'L',
+                       index_type == LSTOPO_INDEX_TYPE_PHYSICAL ? targets[i]->os_index : targets[i]->logical_index,
+                       (unsigned long long) values[j]);
+                output_memattr_initiator(loutput, &initiators[j]);
+                printf("\n");
+              }
+            }
+          }
+          free(initiators);
+          free(values);
+        }
+      }
+    }
+    free(targets);
+  }
+}
+
 int
 output_console(struct lstopo_output *loutput, const char *filename)
 {
@@ -266,6 +394,10 @@ output_console(struct lstopo_output *loutput, const char *filename)
     output_distances(loutput);
     return 0;
   }
+  if (loutput->show_memattrs_only) {
+    output_memattrs(loutput);
+    return 0;
+  }
 
   /*
    * if verbose_mode == 0, only print the summary.
@@ -288,6 +420,7 @@ output_console(struct lstopo_output *loutput, const char *filename)
 
   if (verbose_mode > 1 && loutput->show_only == HWLOC_OBJ_TYPE_NONE) {
     output_distances(loutput);
+    output_memattrs(loutput);
   }
 
   if (verbose_mode > 1 && loutput->show_only == HWLOC_OBJ_TYPE_NONE) {
